@@ -232,6 +232,7 @@ static int get_release(struct release_t *release, const char *name) {
 	FILE *f = NULL;
 	char *hostname, *directory;
 	char line[80];
+	char *p;
 	char buf[SUITE_LENGTH];
 
 	hostname = add_protocol("hostname");
@@ -243,6 +244,13 @@ static int get_release(struct release_t *release, const char *name) {
 	free(directory);
 	directory = strdup(debconf->value);
 
+	/* remove trailing slashes */
+	for (p = directory + strlen(directory) - 1;
+	     p >= directory && p[0] == '/';
+	     p--) {
+		p[0] = '\0';
+	}
+
 	asprintf(&command, "wget -q %s://%s%s/dists/%s/Release -O - | grep -E '^(Suite|Codename):'",
 		 protocol, hostname, directory, name);
 	di_log(DI_LOG_LEVEL_DEBUG, "command: %s", command);
@@ -479,12 +487,14 @@ static int choose_country(void) {
 			debconf_set(debconf, DEBCONF_BASE "country", country);
 		}
 	} else {
-		country = debconf->value;
+		country = strdup(debconf->value);
 	}
 
 	/* Ensure 'country' is set to something. */
-	if (country == NULL || *country == 0)
-		country = "US";
+	if (country == NULL || *country == 0) {
+		free(country);
+		country = strdup("US");
+	}
 
 	char *countries;
 	countries = add_protocol("countries");
@@ -632,7 +642,7 @@ static int set_proxy(void) {
 
 	debconf_get(debconf, px);
 	if (debconf->value != NULL && strlen(debconf->value)) {
-		if (strchr(debconf->value, '://')) {
+		if (strstr(debconf->value, "://")) {
 			setenv(proxy_var, debconf->value, 1);
 		} else {
 			char *proxy_value;
@@ -708,29 +718,33 @@ int set_codename (void) {
 	char *suite;
 	int i;
 
-	/* As suite has been determined previously, this should not fail */
-	debconf_get(debconf, DEBCONF_BASE "suite");
-	if (strlen(debconf->value) > 0) {
-		suite = strdup(debconf->value);
+	/* If preseed specifies codename, omit the codename check */
+	debconf_get(debconf, DEBCONF_BASE "codename");
+	if (! strlen(debconf->value)) {
+		/* As suite has been determined previously, this should not fail */
+		debconf_get(debconf, DEBCONF_BASE "suite");
+		if (strlen(debconf->value) > 0) {
+			suite = strdup(debconf->value);
 
-		for (i=0; releases[i].name != NULL; i++) {
-			if (strcmp(releases[i].name, suite) == 0 ||
-			    strcmp(releases[i].suite, suite) == 0) {
-				char *codename;
+			for (i=0; releases[i].name != NULL; i++) {
+				if (strcmp(releases[i].name, suite) == 0 ||
+				    strcmp(releases[i].suite, suite) == 0) {
+					char *codename;
 
-				if (releases[i].status & GET_CODENAME)
-					codename = releases[i].name;
-				else
-					codename = releases[i].suite;
-				debconf_set(debconf, DEBCONF_BASE "codename", codename);
-				di_log(DI_LOG_LEVEL_INFO,
-					"suite/codename set to: %s/%s",
-					suite, codename);
-				break;
+					if (releases[i].status & GET_CODENAME)
+						codename = releases[i].name;
+					else
+						codename = releases[i].suite;
+					debconf_set(debconf, DEBCONF_BASE "codename", codename);
+					di_log(DI_LOG_LEVEL_INFO,
+						"suite/codename set to: %s/%s",
+						suite, codename);
+					break;
+				}
 			}
-		}
 
-		free(suite);
+			free(suite);
+		}
 	}
 
 	return 0;
